<?php

declare(strict_types=1);

namespace App\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity
 * @ORM\Table(name="data_change_log")
 */
class DataChangeLog
{
    /**
     * @var int|null
     * @ORM\Column(type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private ?int $id = null;

    /**
     * @var DataChangeSource
     * @ORM\ManyToOne(targetEntity="App\Entity\DataChangeSource", inversedBy="sources")
     * @ORM\JoinColumn(name="source")
     */
    public DataChangeSource $source;

    /**
     * @var DataAtom
     * @ORM\ManyToOne(targetEntity="App\Entity\DataAtom", inversedBy="atoms")
     * @ORM\JoinColumn(name="atom")
     */
    public DataAtom $atom;

    /**
     * @var Device
     * @ORM\ManyToOne(targetEntity="App\Entity\Device", inversedBy="dataChangeLogs")
     * @ORM\JoinColumn(name="id_device", referencedColumnName="id_device")
     */
    public ?Device $device;

    /**
     * @var int
     * @ORM\Column(type="integer", nullable=true)
     */
    public ?int $extraId;

    /**
 * @var string
 * @ORM\Column(type="text", nullable=true)
 */
    public ?string $extraTxt;

    /**
     * @var string
     * @ORM\Column(type="text", nullable=true)
     */
    public ?string $valueTxt;

    /**
     * @var \DateTime
     * @ORM\Column(name="change_timestamp", type="datetime", options={"default": "CURRENT_TIMESTAMP"})
     */
    public \DateTime $time;

    /**
     * EventLog constructor.
     */
    public function __construct(){

        $this->time = new \DateTime();

    }

    /**
     * @return int|null
     */
    public function getId(): ?int {

        return $this->id;

    }

    /**
     * @return DataChangeSource
     */
    public function getSource(): DataChangeSource {

        return $this->source;

    }

    /**
     * @param DataChangeSource $source
     * @return void
     */
    public function setSource(DataChangeSource $source): void {

        $this->source = $source;

    }

    /**
     * @return DataAtom|null
     */
    public function getAtom(): ?DataAtom {

        return $this->atom;

    }

    /**
     * @param DataAtom|null $atom
     * @return void
     */
    public function setAtom(?DataAtom $atom): void {

        $this->atom = $atom;

    }

    /**
     * @return Device|null
     */
    public function getDevice(): ?Device {

        return $this->device;

    }

    /**
     * @param Device|null $device
     * @return void
     */
    public function setDevice(?Device $device): void {

        $this->device = $device;

    }

    /**
     * @return string|null
     */
    public function getValueTxt(): ?string {

        return $this->valueTxt;

    }

    /**
     * @param string|null $value
     * @return void
     */
    public function setValueTxt(?string $value): void {

        $this->valueTxt = $value;

    }

    /**
     * @return int|null
     */
    public function getExtraId(): ?int {

        return $this->extraId;

    }

    /**
     * @param int|null $extraId
     * @return void
     */
    public function setExtraId(?int $extraId): void {

        $this->extraId = $extraId;

    }

}