<?php

declare(strict_types=1);

namespace App\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity
 * @ORM\Table(name="device_stats")
 */
class DeviceStats
{

    const ENTITY_TABLE_NAME = 'device_stats';

    /**
     * @var int
     * @ORM\Column(name="id_device", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private int $id;

    /**
     * @var LivenessTime
     * @ORM\ManyToOne(targetEntity="App\Entity\LivenessTime", inversedBy="devicesStats")
     * @ORM\JoinColumn(name="livetime_class")
     */
    public LivenessTime $liveTime;

    /**
     * @var LivenessMisscount
     * @ORM\ManyToOne(targetEntity="App\Entity\LivenessMisscount", inversedBy="devicesStats")
     * @ORM\JoinColumn(name="livemiss_class")
     */
    public LivenessMisscount $liveMissCount;

    /**
     * @var int|null
     * @ORM\Column(name="timeskew", type="integer", nullable=true)
     */
    public ?int $timeSkew;

    /**
     * @var float|null
     * @ORM\Column(type="float", nullable=true)
     */
    public ?float $voltage;

    /**
     * @var int|null
     * @ORM\Column(type="integer", nullable=true)
     */
    public ?int $temperature;

    /**
     * @var DeviceSyncStatus
     * @ORM\ManyToOne(targetEntity="DeviceSyncStatus", inversedBy="devicesStatus")
     * @ORM\JoinColumn(name="sync")
     */
    public DeviceSyncStatus $syncStatus;

    /**
     * @var int|null
     * @ORM\Column(type="integer", nullable=true)
     */
    public int $uptime;

    /**
     * @var int|null
     * @ORM\Column(name="mem_total", type="integer", nullable=true)
     */
    public ?int $memoryTotal;

    /**
     * @var int|null
     * @ORM\Column(name="mem_avail", type="integer", nullable=true)
     */
    public ?int $memoryAvailable;

    /**
     * @var int|null
     * @ORM\Column(type="integer", nullable=true)
     */
    public ?int $processes;

    /**
     * @var int|null
     * @ORM\Column(name="root_avail", type="integer", nullable=true)
     */
    public ?int $rootSpaceDiskAvailable;

    /**
     * @var float|null
     * @ORM\Column(type="float", nullable=true)
     */
    public ?float $loadavg1;

    /**
     * @var float|null
     * @ORM\Column(type="float", nullable=true)
     */
    public ?float $loadavg5;

    /**
     * @var float|null
     * @ORM\Column(type="float", nullable=true)
     */
    public ?float $loadavg15;

    /**
     * @var \DateTime
     * @ORM\Column(type="datetime", nullable=false, options={"default": "CURRENT_TIMESTAMP"})
     */
    public \DateTime $lastReboot;

    /**
     * @var int|null
     * @ORM\Column(type="integer", nullable=true)
     */
    public ?int $rebootCounter;

    /**
     * @var int|null
     * @ORM\Column(type="integer", nullable=true)
     */
    public ?int $signalStrength;

    /**
     * @var int|null
     * @ORM\Column(type="integer", nullable=true)
     */
    public ?int $signalQuality;

    /**
     * @return int|null
     */
    public function getId(): ?int {

        return $this->id;

    }

    /**
     * @return LivenessTime
     */
    public function getLiveTime(): LivenessTime {

        return $this->liveTime;

    }

    /**
     * @return LivenessMisscount
     */
    public function getLiveMissCount(): LivenessMisscount {

        return $this->liveMissCount;

    }

    /**
     * @return int|null
     */
    public function getTimeSkew(): ?int {

        return $this->timeSkew;

    }

    /**
     * @return float|null
     */
    public function getVoltage(): ?float{

        return $this->voltage;

    }

    /**
     * @return int|null
     */
    public function getTemperature(): ?int {

        return $this->temperature;

    }

    /**
     * @return DeviceSyncStatus
     */
    public function getSyncStatus(): DeviceSyncStatus {

        return $this->syncStatus;

    }

    /**
     * @param DeviceSyncStatus $syncStatus
     * @return void
     */
    public function setSyncStatus(DeviceSyncStatus $syncStatus): void {

        $this->syncStatus = $syncStatus;

    }

    /**
     * @return int|null
     */
    public function getUptime(): ?int {

        return $this->uptime;

    }

    /**
     * @return int|null
     */
    public function getMemoryTotal(): ?int {

        return $this->memoryTotal * 1024;

    }

    /**
     * @return int|null
     */
    public function getMemoryAvailable(): ?int {

        return $this->memoryAvailable * 1024;

    }

    /**
     * @return int|null
     */
    public function getRootSpaceDiskAvailable(): ?int {

        return $this->rootSpaceDiskAvailable * 1024;

    }

}